package com.docus.demo.service;

import com.docus.demo.entity.PowerDept;
import com.docus.demo.entity.Tbasic;
import com.docus.demo.facade.BasicDeptService;
import com.docus.demo.mapper.mysql.PowerDeptMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author YongBin Wen
 * @date 2024/10/25 14:40
 */
@Slf4j
@Service
public class BasicDeptServiceImpl  implements BasicDeptService {
    @Resource
    private PowerDeptMapper powerDeptMapper;
    private final  ReentrantLock HANDLE_DEPT_LOCK=new ReentrantLock();
    @Override
    public void handleDept(List<Tbasic> tbasicList) {
        HANDLE_DEPT_LOCK.lock();
        try {
            List<PowerDept> powerDeptList= powerDeptMapper.list();
            Map<String, PowerDept> powerDeptMap = powerDeptList.stream().collect(Collectors.toMap(PowerDept::getDeptName, Function.identity()));
            long maxDeptId=0L;
            Optional<Long> maxDeptIdOp = powerDeptList.stream().map(PowerDept::getDeptId).max(Comparator.naturalOrder());
            if (maxDeptIdOp.isPresent()) {
                maxDeptId=maxDeptIdOp.get();
            }
            List< PowerDept> addPowerDeptList = new ArrayList<>();
            for (Tbasic tbasic : tbasicList) {
                if(tbasic.getAdmissDeptName() == null){
                    tbasic.setAdmissDeptName("");
                }
                if(tbasic.getDisDeptName() == null){
                    tbasic.setDisDeptName("");
                }

                if (!powerDeptMap.containsKey(tbasic.getAdmissDeptName())) {
                    PowerDept powerDept = new PowerDept();
                    maxDeptId=maxDeptId+1;
                    powerDept.setDeptId(maxDeptId);
                    powerDept.setDeptCode(String.valueOf(maxDeptId));
                    powerDept.setDeptName(tbasic.getAdmissDeptName());

                    powerDeptMap.put(powerDept.getDeptName(),powerDept);
                    addPowerDeptList.add(powerDept);
                }

                if (!powerDeptMap.containsKey(tbasic.getDisDeptName())) {
                    PowerDept powerDept = new PowerDept();
                    maxDeptId=maxDeptId+1;
                    powerDept.setDeptId(maxDeptId);
                    powerDept.setDeptCode(String.valueOf(maxDeptId));
                    powerDept.setDeptName(tbasic.getDisDeptName());

                    powerDeptMap.put(powerDept.getDeptName(),powerDept);
                    addPowerDeptList.add(powerDept);
                }

                PowerDept powerDept1 = powerDeptMap.get(tbasic.getAdmissDeptName());
                tbasic.setAdmissDept(powerDept1.getDeptCode());
                PowerDept powerDept2 = powerDeptMap.get(tbasic.getDisDeptName());
                tbasic.setDisDept(powerDept2.getDeptCode());

            }

            if(!addPowerDeptList.isEmpty()){
                powerDeptMapper.insert(addPowerDeptList);
            }

        }catch (Exception ex){
            log.error("处理基础数据科室信息异常，"+ex.getMessage(),ex);
        } finally {
            HANDLE_DEPT_LOCK.unlock();
        }

    }


}
